/**
 * An example ldns program
 *
 * Setup a resolver
 * Query a nameserver
 * Print the result
 */

#include <config.h>
#include <ldns/resolver.h>
#include <ldns/dname.h>        
#include <ldns/host2str.h>

int
main(int argc, char **argv)
{       
        ldns_resolver *res;
        ldns_rdf *qname;
        ldns_rdf *nameserver;
        ldns_pkt *pkt;
	ldns_rr_list *bla;
	ldns_rr *RR;
        const char *nameserver_address = "127.0.0.1";

	if (argc >= 2) {
		nameserver_address = argv[1];
	}
	
        /* init */
        res = ldns_resolver_new(); 
        if (!res)
                return -1;

	RR = ldns_rr_new_frm_str("miek.nl. 3600 IN MX 10 elektron.atoom.net.");
	ldns_rr_print(stdout, RR);
	printf("\n");
	RR = ldns_rr_new_frm_str("miek.nl.   3600   IN   MX  (\n\t10\n\telektron.atoom.net.\n\t)");
	ldns_rr_print(stdout, RR);
	printf("\n");

        nameserver = ldns_rdf_new_frm_str(LDNS_RDF_TYPE_A, nameserver_address);
        if (ldns_resolver_push_nameserver(res, nameserver) != LDNS_STATUS_OK) {
		printf("error push nameserver\n");
		return -1;
	}
        qname = ldns_dname_new_frm_str("open.nlnetlabs.nl.");
        pkt = ldns_resolver_send(res, qname, LDNS_RR_TYPE_A, 0, LDNS_RD);
	if (pkt) {
		ldns_pkt_print(stdout, pkt);
	}
	ldns_pkt_free(pkt);

        pkt = ldns_resolver_send(res, qname, LDNS_RR_TYPE_A, 0, LDNS_RD);
	if (pkt) {
		ldns_pkt_print(stdout, pkt);
		bla = ldns_pkt_authority(pkt);

		ldns_rr_list_print(stdout, bla);

		ldns_rr_list_sort(bla);
		
		printf("sorted\n");
		ldns_rr_list_print(stdout, bla);
	}
	
        return 0;
}
